"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const removeOpSize = JSON.stringify({
    op: "remove",
    path: ""
}).length;
const replaceOpSize = JSON.stringify({
    op: "replace",
    path: "",
    value: ""
}).length - 2;
function isEmpty(data) {
    return _.isString(data) || _.isEmpty(data);
}
function getPatchPath(path) {
    if (path.length === 0) {
        return "";
    }
    else {
        let res = "";
        path.forEach((value) => {
            res += "/" + value;
        });
        return res;
    }
}
function pathSize(path) {
    return getPatchPath(path).length;
}
function removeSize(path) {
    return removeOpSize + pathSize(path) + 1;
}
function replaceSize(path, objSize) {
    return replaceOpSize + pathSize(path) + objSize + 1;
}
function valueSize(value) {
    if (_.isObject(value)) {
        return 2;
    }
    else if (_.isString(value)) {
        return value.length + 2;
    }
    else {
        return String(value).length;
    }
}
function getType(value) {
    if (_.isArray(value)) {
        return "ARRAY";
    }
    else if (_.isObject(value)) {
        return "OBJECT";
    }
    else {
        return "VALUE";
    }
}
function makeTree(oldObj, newObj, path = []) {
    if (newObj === undefined) {
        return {
            path,
            size: 0,
            diffSize: removeSize(path),
            operation: "remove"
        };
    }
    if (isEmpty(newObj)) {
        const size = valueSize(newObj);
        if (newObj === oldObj) {
            return {
                path,
                size,
                diffSize: 0,
                operation: "none"
            };
        }
        else {
            return {
                path,
                size,
                diffSize: replaceSize(path, size),
                operation: "replace"
            };
        }
    }
    if (isEmpty(oldObj) || getType(newObj) !== getType(oldObj)) {
        const size = JSON.stringify(newObj).length;
        return {
            path,
            size,
            diffSize: replaceSize(path, size),
            operation: "replace"
        };
    }
    const keys = _.uniq(_.keys(newObj).concat(_.keys(oldObj)));
    let size = 0;
    let children = [];
    let diffSize = 0;
    keys.forEach((key) => {
        const child = makeTree(oldObj[key], newObj[key], path.concat(key));
        size += child.size + key.length + 2;
        diffSize += child.diffSize;
        children.push(child);
    });
    const replaceDiffSize = replaceSize(path, size);
    if (diffSize > replaceDiffSize) {
        return {
            path,
            size,
            diffSize: replaceDiffSize,
            operation: "replace"
        };
    }
    else {
        return {
            path,
            size,
            diffSize,
            children,
            operation: "children"
        };
    }
}
function get(newObj, path) {
    if (path.length === 0) {
        return newObj;
    }
    else {
        return _.get(newObj, path);
    }
}
function makeDiff(result, newObj, node) {
    if (node.operation === "replace") {
        result.push({
            op: "replace",
            path: getPatchPath(node.path),
            value: get(newObj, node.path)
        });
    }
    else if (node.operation === "remove") {
        result.unshift({
            op: "remove",
            path: getPatchPath(node.path)
        });
    }
    else if (node.operation === "children") {
        node.children.forEach((child) => {
            makeDiff(result, newObj, child);
        });
    }
}
function diff(oldObj, newObj) {
    const result = [];
    const node = makeTree(oldObj, newObj);
    const obj = JSON.parse(JSON.stringify(newObj));
    makeDiff(result, obj, node);
    return result;
}
exports.diff = diff;
//# sourceMappingURL=index.js.map