"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const RandomObject = require("./RandomObject");
const RandomSeed = require("random-seed");
const defaultSpec = {
    seed: "1",
    distributions: {
        noChange: 30,
        add: 1,
        remove: 2,
        replace: 1
    },
    replaceSpec: {
        nodeCountRange: {
            min: 1,
            max: 10
        }
    }
};
function makeChanges(obj, spec, random) {
    const toRemove = [];
    _.each(obj, (value, key) => {
        const operation = RandomObject.selectRandomKey(spec.distributions, random);
        switch (operation) {
            case "replace": {
                const newValue = RandomObject.create(spec.replaceSpec);
                obj[key] = newValue;
                break;
            }
            case "remove": {
                toRemove.push(key);
                break;
            }
            case "add": {
                const newValue = RandomObject.create(spec.replaceSpec);
                if (_.isArray(obj)) {
                    obj.splice(Number(key) + 1, 0, newValue);
                }
                else {
                    obj["new_" + key] = newValue;
                }
            }
            default: {
                if (_.isObject(value)) {
                    makeChanges(value, spec, random);
                }
            }
        }
    });
    if (_.isArray(obj)) {
        toRemove.forEach((key) => obj.splice(Number(key), 1));
    }
    else {
        toRemove.forEach((key) => delete obj[key]);
    }
}
function modify(obj, partialSpec) {
    const spec = _.cloneDeep(defaultSpec);
    _.assign(spec, partialSpec);
    const random = RandomSeed.create(spec.seed);
    makeChanges(obj, spec, random);
}
exports.modify = modify;
//# sourceMappingURL=RandomChanges.js.map